#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 29 17:29:06 2018

@author: Zoe Rehder
"""

import numpy as np
cimport numpy as np

cimport cython

ctypedef np.double_t DTYPE_t
# ----------------------------------
    
cdef hydraulic_con(depth_min, depth_max):
    """
    Parameters
    ----------
    depth : soil depth [m]

    Returns
    -------
    K : hydraulic conductivity  [m s-1]    
    """
    cdef int n = int((depth_max-depth_min)*500) # Steps 
    cdef np.ndarray[DTYPE_t, ndim=1] steps
    if n > 0:
        steps = np.linspace( depth_min, depth_max, n)
    else:
        steps = np.asarray([depth_min,depth_max])
        
    # Hydraulic conductivity [m/s]
    return np.mean(2.5e-04 + 2.5e-04 * np.tanh((steps-5.2e-02 )/-2.3e-02 ))
    
cpdef hydrology(P, E, TD_rim, TD_wb, A_wb, A_veg, A_bare, h_veg, h_bare, WT_wb, topography_super_wb, topography_alpha, del_time):
    """
    Input:
        P                   Precipitation            [m]
        E                   Evaporation              [m]  
        TD_rim              Thaw depth along shore   [m]
        TD_wb               Thaw depth in waterbody  [m]
        A_wb                Total pond area          [m2]  
        A_veg               Total overgrown area     [m2]
        A_bare              Total open water area    [m2]
        h_veg               Mean water depths of veg [m]
        h_bare              Mean water depths of bare[m]
        WT_wb               Watertable below max     [m]  
        topography_super_wb Max water depths         [m]
        topography_alpha    Inclination of pond      [m]
        del_time            Time step length         [s]
        
    Output:
        A_wb_n      Total pond area          [m2]  
        A_veg_n     Total overgrown area     [m2]
        A_bare_n    Total open water area    [m2]
        h_veg_n     Mean water depths of veg [m]
        h_bare_n    Mean water depths of bare[m]
        WT_wb_n     Watertable below max     [m]  
    """
    
#%%        
    cdef double WT_wb_n = WT_wb+(-P+E)
    
    # based on VIC model Gao et al.
    # Runoff = 4/3 * coefficient of discharge (0.94)
    #              * (pi*grav. acceleration)^0.5 * Area
    #              * del_z^3/2 * seconds per timestep
    cdef double del_z, Runoff
    if WT_wb_n < 0 :
        del_z =  - WT_wb_n
        Runoff = min(del_z, 5.67* del_z**(3/2) * del_time / np.sqrt(A_wb) )
    else :
        Runoff = 0   
        
    if WT_wb_n<TD_rim :
        del_z = TD_rim - WT_wb_n # height of unfrozen waterbody rim [m]
        Runoff += hydraulic_con(WT_wb_n, TD_rim) * del_z**2 * del_time * np.sqrt(np.pi/A_wb)
    
    WT_wb_n    += Runoff
    
    if WT_wb_n>TD_wb + topography_super_wb:          # check for too low WT
        print('TD_lim')
        WT_wb_n    = TD_wb + topography_super_wb
#%%
    cdef double tan_a       = np.tan(topography_alpha)
    cdef double del_r      = (WT_wb-WT_wb_n)/tan_a
    cdef double A_wb_n     = np.pi*(np.sqrt(A_wb/np.pi) + del_r)**2
    cdef double A_veg_n    = max(0,A_veg + A_wb_n - A_wb)
    cdef double A_bare_n   = max(0,A_wb_n - A_veg_n)
    if A_bare_n < 1E-3:
        A_bare_n = 0
        A_veg_n  = A_wb_n
    cdef double R_max, R_min, del_V_temp, del_V
    cdef double h_bare_n, h_veg_n
    if topography_super_wb < WT_wb_n: # No standing water
        h_bare_n = 0
        h_veg_n = 0
        A_veg_n = 0
        A_bare_n = A_wb
        A_wb_n = A_wb
    else:    
        if A_bare != 0:
            if (A_bare - A_bare_n) < 1E-5:
                h_bare_n = max(0,h_bare + WT_wb -WT_wb_n) 
                if A_veg_n == 0:
                    h_veg_n = 0
                else: # Compute area of empty, truncated cone --> Volume change
                    R_min = np.sqrt(A_bare_n/np.pi)
                    R_max = np.sqrt(A_wb_n/np.pi)
                    h_veg_n  = (np.pi* tan_a/3 * (R_max**3 - 3*R_max*R_min**2 + 2*R_min**3))/A_veg_n
            else: # Compute volume change for h_veg and h_bare
                R_max       = np.sqrt(max(A_wb,A_wb_n)/np.pi)
                R_min       = max(0,R_max - np.abs(del_r))
                del_V       = np.pi* tan_a/3 * (R_max**3 - R_min**3) #absolute volume change
                # radius change over veg ground, pos sign: A_veg increases
                if A_veg_n == 0:
                    h_veg_n = 0
                    h_bare_n = max(0,(A_bare*h_bare+np.sign(del_r)*(del_V -  h_veg*A_veg))/A_bare_n)
                else:
                    R_min = np.sqrt(max(A_bare,A_bare_n)/np.pi)
                    del_V_temp = np.sign(del_r)*np.pi* tan_a/3 *(R_max**3 - 3*R_max*R_min**2 \
                             + 2*R_min**3) # Change in water volume over veg ground
                    h_veg_n = (h_veg*A_veg + del_V_temp)/A_veg_n
                    h_bare_n = (A_bare*h_bare+del_V-del_V_temp)/A_bare_n
        else:
            R_max       = np.sqrt(max(A_wb,A_wb_n)/np.pi)
            R_min       = max(0,R_max - np.abs(del_r))
            del_V       = np.pi* tan_a/3 * (R_max**3 - R_min**3) #absolute volume change
            h_veg_n  = (A_veg*h_veg+np.sign(del_r)*del_V)/A_veg_n
            h_bare_n = 0
     
    return  A_wb_n, A_veg_n, A_bare_n,  h_veg_n,  h_bare_n,  WT_wb_n