#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar 13 13:48:01 2019

@author: zoerehder

Initialize hydrology and topography for a range of depth and area
"""
import numpy as np
import xarray as xr

def set_overgrownfraction(h_w_veg, alpha, A_wb, h_water):
    """
    Computes overgrown and bare fraction of a tpond shaped like a truncated cone for a given max. vegetation depth
    
    Parameters
    ----------
    h_w_veg : max. vegetation depth
    alpha   : angle between rim and water table
    A_wb    : total area of water body
    h_water : water depth

    Returns
    -------
    A_bare : bare area
    A_veg  : overgrown area
    h_bare : average height of bare part
    h_veg  : average height of overgrown part
    """
        
    tan_a   = np.tan(alpha) # h/del_r
    
    # Assuming pond is so deep, that a part is plant free: A_bare = pi * (r_wb - del_r_veg)^2
    A_bare = np.pi*(np.sqrt(A_wb/np.pi) - h_w_veg/tan_a)**2
    # if pond is shallow: A_veg = A_wb, else: A_veg = A_wb-A_bare
    A_veg  = np.where(h_water<h_w_veg, A_wb, A_wb - A_bare )
    # correct A_bare for A_veg
    A_bare = A_wb - A_veg
    
    # Radii 
    R_wb  =  np.sqrt(A_wb/np.pi)
    R_bare = np.sqrt(A_bare/np.pi)
    R_sup  = np.maximum(0,R_wb - h_water/tan_a)  #Radius of flat bottom

    # Volume of cylinder (pi * r^2 * h, with h = tan_a * r)
    V_mid  = np.maximum(0,np.pi * R_sup**2 * h_water)
    # Volume of a truncated cone (h*pi/3 * (r1^3-r2^3), with h1/2 = tan_a * r1/2) - cylinder volume
    V_edge = np.pi* tan_a/3 * (R_wb**3 - 3*R_wb*R_sup**2 + 2*R_sup**3)
    
    # If all is overgrown, whole volume is V_veg, else same as V_edge with R_bare for the middle cylinder
    V_veg  = np.where(A_bare == 0, V_mid + V_edge, np.pi* tan_a/3 * (R_wb**3 - 3*R_wb*R_bare**2 + 2*R_bare**3))
    V_bare = V_mid + V_edge - V_veg
    
    # mean height = Volume / Area
    h_veg  = V_veg/A_veg
    h_bare = np.nan_to_num(V_bare/A_bare)
    
    return A_bare, A_veg, h_bare, h_veg
    
    
def initialize_topo_hydro(time, A_wb =np.asarray([58,56,1305]), d_bare = np.asarray([0.8,0.6,1.2]), pondtype = np.asarray(['iw', 'pc', 'mp']), reduced_depth = 0):
    """
    Input:
        time                datetime-array with all timesteps
        A_wb        [m2]    area of waterbod
        d_bare      [m]     waterdepth
    Output:
        topography  xarray with fields edges, alpha, super_wb, super_rim, depth_wb
        hyd_output  array for output with initialized first state 
    """
    
    if reduced_depth != 0:
        d_bare *= 1-reduced_depth/100
    
    alpha = np.zeros(len(pondtype))
    alpha[pondtype == 'iw'] = 0.2988
    alpha[pondtype == 'pc'] = 0.3553
    alpha[pondtype == 'mp'] = 0.1997
#%%    
    num_days = len(time)
    npoints  =  len(A_wb)
    ones     = np.ones(( npoints, num_days))
    d_veg    = np.where(d_bare<= 0.5, d_bare,0.5)
    A        = np.asarray(A_wb) * 1.5

    topography = xr.Dataset({ \
                              'A'        : ('waterbody', A                                                                 ,\
                                          {'longname':'Area of each waterbody A = Arim + A_h', 'unit': 'm2'})     ,\
                             'alpha'    : ('waterbody', alpha                  ,\
                                          {'longname':'angle between Rims and W_tab', 'unit': 'RAD'} )                  ,\
                             'super_wb': ('waterbody', 0.1+np.asarray(d_bare),\
                                          {'longname':'surface below the elevated rims', 'unit': 'm'}) ,\
                             'super_rim': ('waterbody', np.zeros(npoints)                                                 ,\
                                          {'longname':'rim surface distribution', 'unit': 'm'} )                 ,\
                             'depth_wb': ('waterbody', 0.1+np.asarray(d_bare),
                                          {'longname':'Depth of waterbodys', 'unit': 'm'} )}                              ,\
                             coords = {'waterbody': range(npoints)})
    
    hyd_output = xr.Dataset({ \
                            'A_wb'         : (['waterbody','time'], (ones *np.array(A_wb)[:,None]   )                   ,\
                                               {'longname':'Area of center of each waterbody', 'unit': 'm2'})     ,\
                            'A_veg'         : (['waterbody','time'], np.zeros((npoints, num_days))                        ,\
                                               {'longname':'Vegetated area of each waterbody', 'unit': 'm2'})     ,\
                            'A_bare'        : (['waterbody','time'], np.zeros((npoints, num_days))                        ,\
                                               {'longname':'Bare area of each waterbody (no vegetation)', 'unit': 'm2'}) ,\
                            'h_veg'         : (['waterbody','time'], (ones * d_veg[:,None] )                     ,\
                                               {'longname':'Mean height of vegetated area', 'unit': 'm2'})              ,\
                            'h_bare'        : (['waterbody','time'], (ones * d_bare[:,None]  )                      ,\
                                               {'longname':'Mean height of bare area', 'unit': 'm2'})                   ,\
                            'open_w'        : (['waterbody','time'], np.zeros((npoints, num_days))                        ,\
                                               {'longname':'Area of open water of each waterbody', 'unit': 'm2'}) ,\
                            'WT_wb'        : (['waterbody','time'], np.zeros((npoints, num_days))                        ,\
                                               {'longname':'Water table at each waterbody', 'unit': 'm'})}                      ,\
                            coords = {'waterbody': range(npoints), 'time':time})
#%%    
#    """ Initilizing output  """
    hyd_output['WT_wb'].values[:,0] = topography.super_wb.values-d_bare

    # at initilzation --> larger WT because not directly after spring.
    h_w_veg = 0.50
    hyd_output['A_bare'].values[:,0],\
    hyd_output['A_veg'].values[:,0], \
    hyd_output['h_bare'].values[:,0],\
    hyd_output['h_veg'].values[:,0] = set_overgrownfraction(h_w_veg,topography.alpha, A_wb, d_bare)
#%%    
    return topography, hyd_output









