#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Oct 20 14:19:00 2020

@author: zoerehder

Methane module of MeEP
"""


import numpy as np
cimport numpy as np
import warnings
import matplotlib.pyplot as plt
cimport cython
from libcpp cimport bool

DTYPE = np.double
ctypedef np.double_t DTYPE_t
ctypedef np.long_t DTYPE_long
@cython.boundscheck(False)
@cython.wraparound(False)


cpdef c_Henry(T, gas):
    """
    Input variables:
    T               :  Temperature, K
    gas             :  either 'CH4', 'O2' or 'N2'
    
    Output variable:
    c_Henry         : mol m-3 Pa-1
    """
    
    cdef double T_ref               = 298.15    # The reference temperature for Henry constants, K
    cdef double c0_Henry_ch4        = 1.4E-5    # Henry constant for methane at the reference temperature, mol/(m**3*Pa)
    cdef double c0_Henry_o2         = 1.3E-5    # Henry constant for oxygen at the reference temperature, mol/(m**3*Pa)
    cdef double c0_Henry_n2         = 6.4E-6    # Henry constant for nitrogen at the reference temperature, mol/(m**3*Pa)
    cdef double T_dependence_ch4    = 1.6E3     # The temperature dependence of Henry constant for methane, K
    cdef double T_dependence_o2     = 1.5E3     # The temperature dependence of Henry constant for oxygen, K
    cdef double T_dependence_n2     = 1.3E3     # The temperature dependence of Henry constant for nitrogen, K
    
    if gas == 'CH4':
        return c0_Henry_ch4 * np.exp(T_dependence_ch4*(1/T - 1/T_ref))
    elif gas == 'O2':
        return c0_Henry_o2 * np.exp(T_dependence_o2*(1/T - 1/T_ref))
    elif gas == 'N2':
        return c0_Henry_n2 * np.exp(T_dependence_n2*(1/T - 1/T_ref))
    else:
        raise(ValueError('This gas is not available'))


cdef methane_fluxes(H, h_s, w_dd, u_star, u, p_a, T_ml, T_b, cCH4_ml_eq, cO2_ml,  LAI, por, P_CH4, gammas, e_a, 
                    g, CH4_frac, c_Henry_b, V_max, k_CH4, k_O2, c_gasexchange, openwater = True):
    """
    Input variables:
    H           :  Mean depth water column, m
    h_s         :  Thickness of bottom sediments layer, m
    w_dd        :  Convective Deardorff velocity scale in the mixed layer, m/s
    u_star      :  Friction velocity scale in the mixed layer, m/s
    u           :  Wind speed, m/s
    p_a         :  Atmospheric pressure, Pa
    T_ml        :  Mixed-layer temperature, Kelvin
    T_b         :  Bottom temperature, Kelvin
    t           :  timestep length, s
    h_ice       :  Thickness of ice, m
    del_ice     :  change in ice thickness since last time step, m
    cO2_ml      :  Oxygen concentration in the mixed layer, [mol m-3]
    cCH4_ml     :  Methane concentration in the mixed layer  [mol m-3]
    stored_CH4  :  methane accumulating under the ice in winter, [mol m-2]
    LAI         :  Leaf-area index, if False: no vegetation considered
    por         :  Porosity of top sediment [-]
    P_CH4       :  Methane productivity [mol m-2 s-1]
    gammas      :  Tuning variable for ebullition [-]
    e_a         :  Gas-filled porosity [m3 m-3]
    g           :  Gravitational acceleration [m s-2]
    CH4_frac    :  Fraction of CH4 in atmosphere [-]
    c_Henry_b   :  Henry's constant at sediment-water interface [mol m-3 Pa-1]
    V_max       :  Maximum methane oxidation rate [mol m-3 s-1] 
    k_CH4       :  half-saturation constant for methane [mol m-3] 
    k_O2        :  half-saturation constant for oxygen [mol m-3] 
    c_gasexchange: Methane gas exchange constant, m/s
    
    Output variables:
    cCH4_ml       :  Methane concentration in the mixed layer, mol/m**3
    cO2_ml        :  Oxygen concentration in the mixed layer, mol/m**3
    F_diff_surf   :  Surface diffusive methane flux to the atmosphere, [mol m-2 s-1]
    F_bubble      :  Bubble methane flux to the atmosphere, [mol m-2 s-1]
    F_plant       :  Plant-mediated methane flux to atmosphere, [mol m-2 s-1]
    """
    cdef:
        double rho_w           = 1E3    # Reference water density, kg/m**3
        double n2_atm0         = 4.96E-1# Concentration of nitrogen in water, in equilibrium with
                                        # atmospheric partial nitrogen pressure (80046 Pa), 
                                        # according to Henry law with reference constant, mol/m**3
        double n2_exp_decay    = 5      # The decay rate in exponential law for nitrogen conecntration in soil, m**(-1)
        double R               = 8.3144598#  ideal gas constant 	[m3 Pa K−1 mol−1]
        double tortuosity_coef = 0.66     # The tortuosity of soil pores
        double D0_liq          = 1.5E-9   # [m2 s-1], Diffusion coefficient for CH4 in the water at 25C - Arah and Stephen (1998)
        double D0_gas          = 1.889E-5 # [m2 s-1], Diffusion coefficient for CH4 in the air at 0C - Arah and Stephen (1998)          
    
    # --- constants for plant-mediated transport
    cdef:
        double  dveg  = 0.1            # density of plant stands 
        double  rk    = 0.01 / 60**2   # [s-1] rate constant, Walter and Heimann (2000)
        int     tveg  = 10             # [-] tundra-specific measure: gas conducting properties of plants (Walter and Heimann 2001)
    cdef double pox   = 0.2            # (Turner et al., 2020, Ström et al., 2005)

    p_hydro_b  = p_a + (H*rho_w*g)          # [Pa]

    cdef double cCH4_b_sat = por * c_Henry_b * gammas * (p_hydro_b -  n2_atm0 * np.exp(-n2_exp_decay*h_s/2)/c_Henry( T_b, 'N2'))
       
    # --- Methane oxidation constant
    # -1- in mixed layer
    V_ml = V_max * cO2_ml / (k_O2+cO2_ml) # [mol m-3 s-1] 
    
    alpha_CH4 = c_Henry_b * R * T_b             # [/] 
     
    # from Sabrekov 2017
    D_liq = tortuosity_coef * (por-e_a) * D0_liq * ((T_b-0.15)/298)**(1.82)   # [m2 s-1] 
    D_gas    = D0_gas * e_a**3.3 / por**2 * ((T_b-0.15)/273)**1.82            # [m2 s-1]
    diffsoil = (por - e_a)*D_liq + e_a/alpha_CH4 * D_gas                      # [m2 s-1]
   
    # plant-mediated transport
    if openwater == False:      
        cplant     = dveg*tveg*rk*LAI * cCH4_b_sat * h_s  # [mol m-2 s-1]
        F_plant    = (1-pox) * min(P_CH4,cplant)
        P_CH4     -= min(P_CH4,cplant)
    else:
        F_plant = 0

    cdef double tmp1, tmp2, tmp3
    cdef double F_bubble, F_b_diff #= 0

    tmp1 = (diffsoil*cCH4_b_sat/(0.5*h_s) + c_gasexchange*cCH4_ml_eq)/(H*V_ml) # [-]
    tmp2 = (diffsoil/(0.5*h_s) + c_gasexchange)/(H*V_ml) # [m3 mol-1]
    tmp3 = tmp1 - k_CH4*tmp2 - 1 # [-] 
    cCH4_ml = (tmp3 + np.sqrt(tmp3**2 + 4*k_CH4*tmp1*tmp2))/(2*tmp2) #[mol m-3]
    F_b_diff = diffsoil* (cCH4_b_sat - cCH4_ml)  /(0.5*h_s)    
    if F_b_diff > P_CH4:
        F_b_diff = P_CH4
        tmp1 = c_gasexchange*(cCH4_ml_eq -  k_CH4) - H*V_ml + F_b_diff
        tmp2 = 4 * c_gasexchange * k_CH4* (c_gasexchange * cCH4_ml_eq + F_b_diff)
        cCH4_ml = (np.sqrt(tmp1**2 + tmp2) + tmp1)/(2*c_gasexchange)
        
    F_diff_surf = c_gasexchange * (cCH4_ml - cCH4_ml_eq)

    F_bubble = max(0,(P_CH4 - F_b_diff))
    
    if openwater == True:
        return cCH4_ml, F_diff_surf, F_bubble
    else:
        return cCH4_ml, F_diff_surf, F_bubble, F_plant


cpdef methane_main(H_o, H_v, A_o, A_v,  h_s, w_dd, u_star, u, p_a, T_ml, T_b, npp, t, ice_off = False, \
                 h_ice = -9999, del_ice = 0, cCH4_ml_o = 0,cCH4_ml_v = 0, cO2_ml = 0, stored_CH4 = 0, LAI = -1, por = 0.97, 
                 # P0_o = 6.5e-07, P0_v = 13E-7, gammas = 0.92, e_a = 0.015, #old
                    P0_o = 1.8e-07, P0_v = 7.2E-7, gammas = 0.36, e_a = 0.035, #q10 = 2
                   # P0_o = 1.1e-07, P0_v = 4.4E-7, gammas = 0.26, e_a = 0.046, #q10 = 3.4
                 c1_piston = 0.00015,  c2_piston = 0.07, pondtype = 'pc', winter = True):
    """
    Input variables:
    H_o         :  Mean Depth of open-water pond fraction, m
    H_v         :  Mean Depth of vegetated pond fraction, m
    A_o         :  Area of bare pond fraction, m**2
    A_v         :  Area of overgrown pond fraction, m**2
    h_s         :  Thickness of bottom sediments layer, m
    w_dd        :  Convective Deardorff velocity scale in the mixed layer, m/s
    u_star      :  Friction velocity scale in the mixed layer, m/s
    u           :  Wind speed, m/s
    p_a         :  Atmospheric pressure, Pa
    T_ml        :  Mixed-layer temperature, Kelvin
    T_b         :  Bottom temperature, Kelvin
    npp         :  net primary productivity and litter fall, mol m-2
    t           :  timestep length, s
    ice_off     :  Boolean whether or not ice came off in the last timestep
    h_ice       :  Thickness of ice, m
    del_ice     :  change in ice thickness since last time step, m
    cO2_ml      :  Oxygen concentration in the mixed layer, mol/m**3
    cCH4_ml_o   :  Methane concentration in the mixed layer, mol/m**3
    cCH4_ml_v   :  Methane concentration in the mixed layer, mol/m**3
    stored_CH4  :  methane accumulating under the ice in winter, [mol m-2]
    por         :  Porosity of top sediment [-]
    LAI         :  Leaf-area index, if False: no vegetation considered
    
    Output variables:
    cCH4_ml_o     :  Methane concentration in the mixed layer, mol/m**3
    cCH4_ml_v     :  Methane concentration in the mixed layer, mol/m**3
    cO2_ml        :  Oxygen concentration in the mixed layer, mol/m**3
    F_diff_surf_o :  Surface diffusive methane flux to the atmosphere, [mol m-2 s-1]
    F_diff_surf_v :  Surface diffusive methane flux to the atmosphere, [mol m-2 s-1]
    F_bubble_o    :  Bubble methane flux to the atmosphere, [mol m-2 s-1]
    F_bubble_v    :  Bubble methane flux to the atmosphere, [mol m-2 s-1]
    F_plant       :  Plant-mediated methane flux to atmosphere, [mol m-2 s-1]
    stored_CH4    :  methane accumulating under the ice in winter, [mol m-2]
    """
    cdef:
        double K_to_C = 273.15 # Kelvin equal 0°C
        double c1_Sc =  1909.4     
        double c2_Sc = -120.78     
        double c3_Sc =  4.1555     
        double c4_Sc = -0.080578   
        double c5_Sc =  0.00065777
        double nu_w = 1.307E-6             # Molecular viscosity of water at T = 10 C, m**2/s 
        double CH4_frac = 1.9499E3* 1E-9   # ppb * E-9 = Fraction --- mean of 'SAM_cCO2_cCH4.csv'
        double O2_frac  = 0.19             # The ratio of atmospheric oxygen pressure to total atmospheric pressure, n/d  
        double c_w             = 3990.   # Specific heat of water,  J/(kg*K)
        double rho_ice         = 0.92E+3 # Reference density of ice, [kg m-3]
        
        # --- constants for methane oxidation constant
        # maximum methane oxidation rate
    cdef:
        double V_max =  1.412E-7# [mol m-3 s-1] Martinez-Cruz (2015) mean for all  lakes
                       
        # half-saturation constant
        double k_CH4 = 0.006875 # [mol m-3] Martinez-Cruz (2015) 
        double k_O2  = 0.0195   # [mol m-3] Martinez-Cruz (2015) 
        
        # --- constants for methane production in sediments
    cdef: 
        double q10  = 2#3.4            # [-] temp. dependency constant
        int alpha   = 20             # [m-1] 
    
    cdef:    
        double g = 9.81 # [m s-2] gravitational acceleration
        double c1_diff_w       = 9.798E-10
        double c2_diff_w       = 2.986E-11
        double c3_diff_w       = 4.381E-13
             

    
    cdef double diff_const = 1
    
    cdef:
        double Ea  = 0.96 # Jansen et al., 2022
        double k_B = 8.62E-5 # eV K-1
        double Tc  = 283.15#285.34#279.210# # K, mean lake bottom temperature as modeled by FLake for 2004-2019 in _hist simulations
        # double a   = 5.3384005982790624e+17 # faktor between tuning

    # --- Methane production in sediments
    cdef double base_P # Dean et al., 2020; Bouchard et al., 2015
    if pondtype == 'mp':
        base_P = 0.25
    elif pondtype == 'pc':
        base_P  = 0.25
    elif pondtype == 'iw':
        base_P = 0.25
        O2_frac /= 2 # account for stratification of iw ponds
    else:
        raise ValueError('Pondtype not implemented')
        
    if T_b >= K_to_C:
# q10
        P_CH4_o = P0_o * q10**((T_b-K_to_C)/(Tc-K_to_C)) / alpha * (1 - np.exp(-alpha*h_s)) * ((1-base_P)*npp * np.tanh(A_v/A_o) + base_P) # [mol m-2 s-1]
        P_CH4_v = P0_v * q10**((T_b-K_to_C)/(Tc-K_to_C)) / alpha * (1 - np.exp(-alpha*h_s)) * ((1-base_P)*npp + base_P) # [mol m-2 s-1]
#Arrhenius Jansen 2022
        # P_CH4_o = P0_o *np.exp(Ea/k_B*((1/Tc)-(1/T_b))) / alpha * (1 - np.exp(-alpha*h_s)) * ((1-base_P)*npp * np.tanh(A_v/A_o) + base_P) # [mol m-2 s-1]
        # P_CH4_v = P0_v *np.exp(Ea/k_B*((1/Tc)-(1/T_b))) / alpha * (1 - np.exp(-alpha*h_s)) * ((1-base_P)*npp + base_P) # [mol m-2 s-1]

    else:
        P_CH4_o = 0
        P_CH4_v = 0

    cdef double alpha_CH4, cCH4_ml, cCH4_ml_eq, cCH4_ml_sat,\
                cplant, c_gasexchange, c_Henry_b, del_cCH4, del_ox, diffsoil, D_liq,\
                D_gas, F_bubble_o, F_bubble_v, F_diff_surf_o,  F_diff_surf_v ,F_plant, k_ox_hypo, k_ox_ml,\
                p_atm, p_hydro_b, S, Sc, tmp1, tmp2, tmp_sin, tmp_cos,\
                H, cCH4_ml_p, P_CH4
    cdef int t_s


# =============================================================================
#     If ice is present  
# =============================================================================
    if h_ice > 0:
        if winter == True:
            H         = max(H_v,H_o)
            cCH4_ml_p = (cCH4_ml_o*A_o + cCH4_ml_v*A_v) /(A_o+A_v)
            P_CH4     = (P_CH4_o*A_o   + P_CH4_v*A_v)   /(A_o+A_v)
            
            if h_ice >= H:
                del_ice = max(0, del_ice-(h_ice - H))
                h_ice = H
                stored_CH4 += P_CH4 * t
                cCH4_ml_o  = 0
                cCH4_ml_v  = 0  
                cO2_ml = 0
                
            else: # there is  fluid water
                H -= min(H,h_ice) # H is the height of fluid water column
                cCH4_ml = np.copy(cCH4_ml_p)
                # ** MAXIMUM SATURATION OF METHANE **
                # Calculation of [CH4]_ml after Peltola, 2018
                # saturation pressure right under the ice sheet -> most likely place for bubbles to form
                cCH4_ml_sat =  (p_a + rho_ice*g*h_ice) * c_Henry(T_ml, 'CH4')    # [mol m-3]
                
                F_diff_surf = 0
                cCH4_ml *= (H+del_ice)/H      # dilute cCH4 - assume there is no methane in formerly frozen water (!!)

                # Production   
                cCH4_ml += P_CH4 * t/H #  [mol m-3] 
                
                # check saturation
                if cCH4_ml_sat <= cCH4_ml:
                    stored_CH4 += (cCH4_ml - cCH4_ml_sat)*H # [mol m-2]
                    cCH4_ml = np.copy(cCH4_ml_sat)
                elif stored_CH4 > 0:
                    del_cCH4    =  min(stored_CH4/H, cCH4_ml_sat-cCH4_ml)
                    cCH4_ml     += del_cCH4
                    stored_CH4  -= del_cCH4 * H
                        
                # Prescribed depletion of oxygen
                # Oxygen is excluded during freeze  https://doi.org/10.1029/2020WR027990
                del_ox = (p_a + rho_ice*g*h_ice)*c_Henry(T_ml, 'O2')
                cO2_ml *= (H+del_ice)/H
                
                cO2_ml  = max(0.0,cO2_ml - 1.447*1E-7*t) # 1.447×10^-7  mol O2 m-3 s-1 = 0.4 mg O2 L-1 d-1 https://aslopubs.onlinelibrary.wiley.com/doi/full/10.1002/lno.10126
                del_ox = 0
                if cO2_ml > 0:
                    # --- Methane oxidation 
                    del_ox = V_max * cO2_ml / (cO2_ml + k_O2) * cCH4_ml / (cCH4_ml + k_CH4) * t # [mol m-3]
                    del_ox = min(del_ox, cO2_ml/2, cCH4_ml) # for every mol of oxidized CH4, 2 mol O2 are needed
                    cO2_ml = max(0,cO2_ml-2*del_ox)
                    cCH4_ml -= del_ox
                
                    if cCH4_ml < cCH4_ml_sat and stored_CH4 > 0:
                        del_cCH4    =  min(stored_CH4/H, cCH4_ml_sat-cCH4_ml)
                        cCH4_ml    += del_cCH4
                        stored_CH4 -= del_cCH4 * H
                if H_o > 0:
                    cCH4_ml_o = cCH4_ml
                if H_v > 0:
                    
                    cCH4_ml_v = cCH4_ml
            
        F_bubble_v = 0
        F_bubble_o = 0
        F_diff_surf_o = 0
        F_diff_surf_v = 0
        F_plant = 0
        if stored_CH4 <= 0:
            stored_CH4 = 1E-19
# =============================================================================
#     If water is open
# =============================================================================
    else:     
        if ice_off == True:
            H         = max(H_v,H_o)
            cCH4_ml_p = (cCH4_ml_o*A_o + cCH4_ml_v*A_v) /(A_o+A_v) *(H+del_ice)/H
        
        del_ox = 0
        # Calculation of [CH4]_ml and [O2]_ml, equilibrated with atm. conc.
        cCH4_ml_eq = CH4_frac * p_a * c_Henry(T_ml, 'CH4')    # [mol m-3]
        cO2_ml     = O2_frac  * p_a * c_Henry(T_ml, 'O2')     # [mol m-3]
        # Schmidt number for methane
        # Wanninkhof 2014
        Sc = c1_Sc + c2_Sc*(T_ml-K_to_C) + c3_Sc*(T_ml-K_to_C)**2+ c4_Sc*(T_ml-K_to_C)**3 + c5_Sc*(T_ml-K_to_C)**4

        if Sc < 0:
            warnings.warn('encountered negative Schmidt Number, set to zero!')
            print(T_ml,Sc)
            Sc = 0
    
        # Piston velocity after Heiskanen et al., 2014
        # Methane gas exchange constant, m/s
        c_gasexchange = diff_const*(np.sqrt((c1_piston* u)**2 + (c2_piston*w_dd)**2) / np.sqrt(Sc))
              
        c_Henry_b = c_Henry(T_b, 'CH4')         # [mol m-3 Pa-1]
        
        if H_o > 0:
            cCH4_ml_o     ,\
            F_diff_surf_o ,\
            F_bubble_o = methane_fluxes(H_o,       
                                        h_s,       w_dd, 
                                        u_star,    u, 
                                        p_a,       T_ml, 
                                        T_b,       cCH4_ml_eq, cO2_ml,  
                                        LAI,       por, 
                                        P_CH4_o,   gammas, 
                                        e_a,       g, 
                                        CH4_frac,  c_Henry_b, 
                                        V_max,     k_CH4, 
                                        k_O2,      c_gasexchange,
                                        openwater = True)
        else:
            F_diff_surf_o = 0
            F_bubble_o = 0
                  
        if H_v > 0:
            cCH4_ml_v     ,\
            F_diff_surf_v ,\
            F_bubble_v    ,\
            F_plant       = methane_fluxes(H_v,       
                                           h_s,       w_dd, 
                                           u_star,    u, 
                                           p_a,       T_ml, 
                                           T_b,       cCH4_ml_eq, cO2_ml,  
                                           LAI,       por, 
                                           P_CH4_v,   gammas, 
                                           e_a,       g, 
                                           CH4_frac,  c_Henry_b, 
                                           V_max,     k_CH4, 
                                           k_O2,      c_gasexchange,
                                           openwater = False)
        else:
            F_diff_surf_v = 0
            F_bubble_v = 0

        if  ice_off == True and winter == True:
            # excess storage
            if H_o > 0:
                F_bubble_o += stored_CH4/t
                if cCH4_ml_p > cCH4_ml_o:
                    tmp1 = cCH4_ml_p*(H+del_ice)/H
                    F_diff_surf_o += (tmp1 - cCH4_ml_o) * H_o/t
            else:
                F_bubble_v += stored_CH4/t
        
            if cCH4_ml_p > cCH4_ml_v:
                tmp1 = cCH4_ml_p*(H+del_ice)/H
                F_diff_surf_v += (tmp1 - cCH4_ml_v) * H_v/t
        stored_CH4 = 0      
     
    return  cCH4_ml_o,  cCH4_ml_v,  cO2_ml, F_diff_surf_o, F_diff_surf_v,\
            F_bubble_o, F_bubble_v, F_plant,  stored_CH4 
            