# cython: profile=True
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan 27 17:16:40 2020

@author: Zoé Rehder, Moritz Langer

Compute Soil Heat Transfer around and below water bodies
"""
#implcit heat transfere with phase change following an approch by:
#C.R. SWAMINATHAN and V.R. VOLLER (1992) in METALLURGICAL TRANSACTION
#this version uses a free water freeze curve, single side lateral heat flux is included

import numpy as np
cimport numpy as cnp
import xarray as xr
import warnings
from matplotlib import pyplot as plt
from pandas import Timestamp

cimport cython
ctypedef cnp.double_t DTYPE_t
ctypedef cnp.long_t DTYPE_long
@cython.boundscheck(False)
@cython.wraparound(False)

##%%--------------------------------------------------------------------------
cpdef TotalC(cnp.ndarray[DTYPE_t, ndim=1] depth, location):
    """
    Input
    -----
    depth : [m] soil depth 
    location : 'tundra' or 'wb'

    Returns
    -------
    TotalC : [frac.] Total organic carbon approximation based on Zubrzycki et al. (2013) 
    """
    if location == 'tundra':
        return 0.19*np.exp(-4.04 *depth)
    elif location == 'wb':
        return 0.2*np.exp(-2.01 *depth) 
    else:
        raise ValueError('Not a valid location')
        
cpdef MakeGrid(Zp,dxp,kp):
    """
    Input
    -----
    Zp  : grid cell midpoints [m]
    dxp : grid cell sizes [m]
    kp  : heat conductivity of cell [W m-1 K-1] 
    
    Returns
    -------
    Zn   : upper edge of grid cell [m]
    Zs   : lower edge of grid cell [m]
    dxn  : grid cell size of next heigher layer [m]
    dxs  : grid cell size of next lower layer [m]
    kn   : upwards conductivity [W m-1 K-1] 
    ks   : downwards conductivity [W m-1 K-1] 
    """
    cdef int N = len(Zp)
    Zn = Zp-dxp/2
    Zs = Zp+dxp/2
    
    dxn=np.ones(N)
    dxs=np.ones(N)
    
    cdef int i = 0
    for i in range(1,N-1):
        dxn[i] = Zp[i] - Zp[i-1]
        dxs[i] = Zp[i+1] - Zp[i]
       
    dxs[0] = Zp[1] - Zp[0]
    dxs[N-1] = dxp[N-1]/2
    
    dxn[0] = dxp[0]/2 
    dxn[N-1] = Zp[N-1] - Zp[N-2]
    
    kn, ks = Conductivities(dxp,dxn,dxs,kp)

    return Zn, Zs, dxn, dxs, kn, ks

cdef Conductivities(cnp.ndarray[DTYPE_t, ndim=1] dxp,
                    cnp.ndarray[DTYPE_t, ndim=1] dxn,
                    cnp.ndarray[DTYPE_t, ndim=1] dxs,
                    cnp.ndarray[DTYPE_t, ndim=1] kp):
    """
    Input
    -----
    dxp  : grid cell sizes [m]
    dxn  : grid cell size of next heigher layer [m]
    dxs  : grid cell size of next lower layer [m]
    kp   : heat conductivity of cell [W m-1 K-1] 
    
    Returns
    -------   
    kn   : upwards conductivity [W m-1 K-1] 
    ks   : downwards conductivity [W m-1 K-1] 
    """    
    cdef int N = len(dxp)
    cdef cnp.ndarray[DTYPE_t, ndim=1] kn=np.ones(N)
    cdef cnp.ndarray[DTYPE_t, ndim=1] ks=np.ones(N)

    kn[1:N-1] = 1/(dxp[1:N-1]/(2*dxn[1:N-1])*kp[1:N-1]**-1 + dxp[:N-2]/(2*dxn[1:N-1])*kp[:N-2]**-1)       
    ks[1:N-1] = 1/(dxp[1:N-1]/(2*dxs[1:N-1])*kp[1:N-1]**-1 + dxp[2:]  /(2*dxs[1:N-1])*kp[2:] **-1)
            
    kn[0] = kp[0]
    kn[N-1] = kp[N-1]
    ks[0] = kp[0]
    ks[N-1] = kp[N-1]
    return kn, ks #[W m-1 K-1] 
    
cpdef ThermalConductivity(cnp.ndarray[DTYPE_t, ndim=1] WaterIce,
                          cnp.ndarray[DTYPE_t, ndim=1] Water,
                          cnp.ndarray[DTYPE_t, ndim=1] SolidMatrix, 
                          cnp.ndarray[DTYPE_t, ndim=1] TOC): # W m-1 K-1
    """
    Input
    -----
    WaterIce    : Water and ice fraction of grid cells [-]
    Water       : Liquid water fraction of grid cells [-]
    SolidMatrix : Soil fraction of grid cells [-]
    TOC         : Organic fraction of soil [-]
    
    Returns
    -------
    kp   : heat conductivity of cell [W m-1 K-1] 
    """
    cdef double ka = 0.025       # air [Hillel(1982)]
    cdef double kw = 0.57        # water [Hillel(1982)]
    cdef double ko = 0.25        # organic [Hillel(1982)]
    cdef double km = 3.8         # mineral [Hillel(1982)]
    cdef double ki = 2.2         # ice [Hillel(1982)]
    
    cdef cnp.ndarray[DTYPE_t, ndim=1] Ice = WaterIce - Water
    cdef cnp.ndarray[DTYPE_t, ndim=1] Air = 1.0 - WaterIce - SolidMatrix
    return (Water* kw**0.5 + Ice* ki**0.5 + (SolidMatrix-TOC)* km**0.5 + TOC* ko**0.5 +Air* ka**0.5)**2.0
    
cpdef HeatCapacity(cnp.ndarray[DTYPE_t, ndim=1] WaterIce, 
                   cnp.ndarray[DTYPE_t, ndim=1] Water, 
                   cnp.ndarray[DTYPE_t, ndim=1] SolidMatrix, 
                   cnp.ndarray[DTYPE_t, ndim=1] TOC):
    """
    Input
    -----
    WaterIce    : Water and ice fraction of grid cells [-]
    Water       : Liquid water fraction of grid cells [-]
    SolidMatrix : Soil fraction of grid cells [-]
    TOC         : Organic fraction of soil [-]
    
    Returns
    -------
    cp   : Heat capacity of grid cells [J m-3 K-1]
    """
    cdef double cw = 4.2E6      #[J/m^3K]  heat capacity water
    cdef double co = 2.5E6      #[J/m^3K]  heat capacity organic
    cdef double cm = 2.0E6      #[J/m^3K]  heat capacity mineral
    cdef double ca = 0.00125E6  #[J/m^3K]  heat capacity pore space
    cdef double ci = 1.9E6      #[J/m^3K]  heat capacity ice
    
    cdef cnp.ndarray[DTYPE_t, ndim=1] air = 1.0 - WaterIce - SolidMatrix 
    cdef cnp.ndarray[DTYPE_t, ndim=1] ice = WaterIce - Water
    return (Water*cw + ice*ci + (SolidMatrix-TOC)*cm + TOC*co + air*ca)


cpdef Enthalpie(cnp.ndarray[DTYPE_t, ndim=1] T, 
                cnp.ndarray[DTYPE_t, ndim=1] Water, 
                cnp.ndarray[DTYPE_t, ndim=1] HC):
    """
    Input
    -----
    T     : Soil temperature [°C]
    Water : Liquid water fraction of grid cells [-]
    HC    : Heat capacity [J m-3 K-1]
        
    Returns
    -------
    H     : Enthalpien [J m-3]
    dHdT  : Derivitive of enthalpie [J m-3 K-1] 
    """ 
    cdef double rho = 999.8  # [kg/m^3]
    cdef double Lsl = 333550 # [J/kg]
    cdef double L   = rho*Lsl# [J/m^3]

    cdef cnp.ndarray[DTYPE_t, ndim=1] H    = T*HC + Water*L # [J/m^3]
    cdef cnp.ndarray[DTYPE_t, ndim=1] dHdT = HC*(T!=0) + 1e8*(T==0) # [J/m^3K] 
    return H, dHdT

cdef EnthalpieInv(cnp.ndarray[DTYPE_t, ndim=1] H, 
                  cnp.ndarray[DTYPE_t, ndim=1] WaterIce,
                  cnp.ndarray[DTYPE_t, ndim=1] HC):
    """
    Input
    -----    
    H           : Enthalpien [J m-3]
    WaterIce    : Water and ice fraction of grid cells [-]
    HC          : Heat capacity [J m-3 K-1]
        
    Returns
    -------
    T     : Soil temperature [°C]
    fl    : Fraction of liquid water of total water content [-]
    """
    cdef double rho = 999.8   # [kg/m^3]
    cdef double Lsl = 333550  # [J/kg]
    cdef double L = rho*Lsl   # [J/m^3]
        
    cdef cnp.ndarray[DTYPE_t, ndim=1] fl = np.logical_and(H>0, H<=L*WaterIce)*(H/(L*WaterIce)) + (H>L*WaterIce)
    cdef cnp.ndarray[DTYPE_t, ndim=1] T = (H>=(L*WaterIce))*((H-L*WaterIce)/HC) + (H<0)*(H/HC)     
    return T, fl

cpdef TDMAsolver(cnp.ndarray[DTYPE_t, ndim=1] a,
                 cnp.ndarray[DTYPE_t, ndim=1] b,
                 cnp.ndarray[DTYPE_t, ndim=1] c,
                 cnp.ndarray[DTYPE_t, ndim=1] d):
    """
    Input
    -----
    a, b, c are the column vectors for the compressed tridiagonal matrix, d is the right vector
    
    Returns
    -------
    x
    """
    cdef int n = len(b) # n is the number of rows
    cdef cnp.ndarray[DTYPE_t, ndim=1] x = np.zeros(n)
    # Modify the first-row coefficients
    c[0] = c[0] / b[0]    # Division by zero risk.
    d[0] = d[0] / b[0]    # Division by zero would imply a singular matrix.
    
    cdef int i
    cdef double temp
    for i in range(1,n-1): 
        temp = b[i] - a[i] * c[i-1]
        c[i] /=  temp
        d[i] = (d[i] - a[i] * d[i-1]) / temp
    
    d[n-1] = (d[n-1] - a[n-1] * d[n-2])/( b[n-1] - a[n-1] * c[n-2])
    
    # Now back substitute.
    x[n-1] = d[n-1]
    for i in range(n-2,-1,-1): 
        x[i] = d[i] - c[i] * x[i + 1]
    return x

cdef SnowCover(Water, WaterIce, Zs, snow_depth, snow_max, snow_fall, SnowDensity, WaterDensity, current_date):
    """
    Input
    -----
    WaterIce        : Water and ice fraction of grid cells [-]
    Water           : Liquid water fraction of grid cells [-]    
    Zs              : lower edge of grid cell [m]
    snow_depth      : Thickness of snow layer [m]
    snow_max        : Upper limit of snow thickness [m]
    snow_fall       : Liquid water equivalent of snow fall [m]
    SnowDensity     : Density of snow [km m-3]
    WaterDensity    : Density of water [km m-3]
    current_date    : Timestamp of timestep [Datetime]
    
    Returns
    -------
    WaterIce        : Water and ice fraction of grid cells [-]
    Water           : Liquid water fraction of grid cells [-]    
    snow_depth      : Thickness of snow layer [m]
    idx             : air-ground interface index [-]
    """
    #get current frozen snow cover dept
    cdef int snow_dw = sum(Zs<=0.0)-1 #lower most snow cell
    cdef int snow_up = max(snow_dw-sum(np.logical_and(WaterIce>0.0 , Zs<=0.0)),0) #cell above last snow cell
    cdef int old_snow_up = snow_up
    cdef double delta_snow_depth = snow_depth - max(-Zs[snow_up],0.0) #the difference between real and grided snow depth
    
    cdef double water_hold = 0.025
    cdef double snow_rm    = 0.5*SnowDensity/WaterDensity 
    cdef double meltwater  = 0.0  # initlize melt term
    cdef double SnowDensity_max = 363 # Maritime
    cdef double k1 = 0.0029 # Parameters for martime enviroments on snow density taken from Sturm et al., 2010
    cdef double k2 = 0.0049
          
    #-- snow depletion
    cdef double water_holding_cap, UTmax
    cdef int i,j,k,new_snow_up, idx
    cdef cnp.ndarray[DTYPE_t, ndim=1] WaterIce_cp, Water_cp   
    
    #-- snow compaction
    cdef double BulkSnowDensity_new, BulkSnowDensity_current, snow_depth_compact
    cdef int delta_doy, DOY_snowseason
    cdef cnp.ndarray[DTYPE_t, ndim=1] WaterIce_current, WaterIce_new    

    if snow_up < snow_dw: # snow cover exists
        # route melt water downward
        for i in np.arange(snow_up+1,snow_dw): # All ice layers except lowermost
            water_holding_cap = water_hold*(WaterIce[i]-Water[i]) #2.5 percent of the water equivalent of the snowpack existing in the ice phase (U.S. Army, 1956; Leaf, 1966)   
            
            #route melt water downward
            if Water[i] > water_holding_cap:
                meltwater       = max(0.0, Water[i]-water_holding_cap)
                WaterIce[i]     = WaterIce[i] - meltwater
                Water[i]        = Water[i] - meltwater
                WaterIce[i+1]   = WaterIce[i+1] + meltwater
                Water[i+1]      = Water[i+1] + meltwater
                    
                #remove snow cell if it contains less ice than half of initial SWE
                if (WaterIce[i]-Water[i])<=snow_rm:
                    #treat rest as additional meltwater
                    meltwater       = WaterIce[i]
                    WaterIce[i]     = 0.0
                    Water[i]        = 0.0    
                    WaterIce[i+1]   = WaterIce[i+1] + meltwater
                    Water[i+1]      = Water[i+1] + meltwater
                    
                    #update upper snow index and snow depth
                    snow_up     = snow_up+1
                    snow_depth  = max(-Zs[snow_up],0.0) + delta_snow_depth
        
        #last snow cell
        i = snow_dw
        #remove last snow cell and/or determine routable water
        if (WaterIce[i]-Water[i])<=snow_rm:
            meltwater = WaterIce[i]
            WaterIce[i] = 0.0
            Water[i] = 0.0     
            #update upper snow indx and snow depth
            snow_up = snow_up+1
            snow_depth = max(-Zs[snow_up],0.0) + delta_snow_depth
            if snow_up == snow_dw:
                # if the snow cover is completely melted set snow_depth to zero
                snow_depth = 0.0
        else:
            meltwater   = Water[i]     
            #consider this water as free routable water for upward routing
            Water[i]    = Water[i]-meltwater
            WaterIce[i] = WaterIce[i]-meltwater
        
        #route meltwater upward 
        if (meltwater>0.0) and (snow_up<snow_dw) :
            #copy water and ice arrays
            WaterIce_cp = np.copy(WaterIce)
            Water_cp    = np.copy(Water)
            j           = np.copy(snow_dw)
            for k in np.arange(snow_dw,snow_up-1,-1): 
                #step over previously removed snow cells
                while WaterIce_cp[j]==0.0 and j>=old_snow_up+1 :
                    j = j-1
                
                #melt water routing
                if j>0 : #only for safety reason
                    WaterIce[k] = WaterIce_cp[j] + meltwater
                    Water[k] = Water_cp[j] + meltwater
                    meltwater = 0.0
                    
                    #further route rest if cell is filled until an uptake
                    UTmax = SnowDensity_max /WaterDensity #1.0 would be completely filled
                    if WaterIce[k]>UTmax :
                        meltwater = WaterIce[k]-UTmax
                        WaterIce[k] = UTmax
                        Water[k] = Water[k]-meltwater
                else: #only for safety reason
                    WaterIce[k] = 0.0
                    Water[k] = 0.0
                j = j-1
    
    #remove all water and ice rests above actual snow cover
    WaterIce[0:snow_up+1] = 0.0
    Water   [0:snow_up+1] = 0.0
    
    #snow compaction module according to Sturm et al.(2010)
    if snow_up < snow_dw: #snow cover exists
        current_year = current_date.year 
        if current_date < Timestamp(current_year,10,1):
            delta_doy = 0
        else:
            delta_doy = Timestamp(current_year,10,1).dayofyear+92
         
        DOY_snowseason = current_date.dayofyear -delta_doy
        DOY_snowseason = max(-92,min(181,DOY_snowseason)) #limints according to Sturm et al. 2010
        BulkSnowDensity_new = (SnowDensity_max - SnowDensity)\
                              * (1.0 - np.exp(-k1*snow_depth - k2*DOY_snowseason))\
                              + SnowDensity
        #current snow bulk density according to gridded ice content (only takes Ice into account)
        BulkSnowDensity_current = max(SnowDensity, WaterDensity*np.mean(WaterIce[snow_up+1:snow_dw+1]))

        if (current_date >= Timestamp(current_year,8,1)) and (current_date < Timestamp(current_year,9,1)):
            snow_depth = 0.0 #set snow depth to zero in the period from August 1 to August 31 to prevent glaciers
        else:
            snow_depth = max(min(snow_depth, snow_max), 0.0); #avoid values smaller 0.0 and larger max snow depth

        #new snow depth according to new snow bulk snow density this emulates compaction only
        snow_depth_compact = snow_depth*BulkSnowDensity_current/BulkSnowDensity_new
        snow_depth_compact = max(min(snow_depth_compact, snow_max), 0.0) #avoid values smaller 0.0 and larger max snow depth
        new_snow_up = np.argmin(abs(snow_depth_compact-(-Zs)))
        #remove snow cover if snow depth is lower than gridded

        if new_snow_up>snow_up:
            WaterIce[snow_up:new_snow_up+1] = 0.0
            Water[snow_up:new_snow_up+1] = 0.0
            snow_up = new_snow_up
            snow_depth = max(-Zs[snow_up],0.0)

            WaterIce_current = WaterIce[snow_up+1:snow_dw+1]
            WaterIce_new = WaterIce[snow_up+1:snow_dw+1]*(BulkSnowDensity_new/BulkSnowDensity_current)
            WaterIce_current = WaterIce_new
            WaterIce_current[WaterIce_current>0.9] = 0.9
            WaterIce[snow_up+1:snow_dw+1] = WaterIce_current

    snow_depth = snow_depth + snow_fall*(WaterDensity/SnowDensity) #increses by snow depth by snow fall
    snow_depth = max(min(snow_depth,snow_max),0.0)
    new_snow_up = np.argmin(abs(snow_depth- -Zs)) 
    #build a new snow cell on top if snow depth fills a complet new cell

    if new_snow_up<snow_up:
        WaterIce[new_snow_up+1:snow_up+1] = SnowDensity/WaterDensity
        Water[new_snow_up+1:snow_up+1] = 0.0
        snow_up = new_snow_up
        
    #get new air-ground interface index
    idx = snow_up+1
        
    return Water, WaterIce, snow_depth, idx

#%%# ========================================================================
#  do not change unless you know what you are doing
cpdef CryoGridImplizit(T_air, I_bot, GRID,STRAT,WaterDepth,STATVAR,TEMP,current_date,snow_fall=0):
#%%# ========================================================================   
    """
    Input
    -----
    T_air                   : Air temperatue [K]
    I_bot                   : Incoming radiation at bottom of the pond []
    GRID,STRAT,STATVAR,TEMP : CryoGridLite dictionaries
    current_date            : Timestamp of timestep [Datetime]
    snow_fall               : Liquid water equivalent of snow fall [m]
        
    Returns
    -------
    T                  : Water temperature [K]
    Water              : Liquid water fraction [-]
    Ice                : Frozen water fraction [-]
    Q_bot              : Bottom heat flux [W m-2]
    TD                 : Thaw depth [m] 
    STRAT,STATVAR,TEMP : CryoGridLite dictionaries
    SnowDepth          : Thickness of snow layer [m]
    """

    # for Q_bot
    cdef double kappa_w     = 5.46E-01    # Molecular heat conductivity of water [J m^{-1} s^{-1} K^{-1}]
    cdef int    Phi_B1      = 2           # B1 shape-function derivative 
    cdef double H_B1_min    = 1.0e-3      # Minimum thickness of the upper layer of bottom sediments [m]
    cdef double T_zero      = 273.15
    
    T_air -= T_zero # Kelvin = 0 Celsius
    
    cdef int    dt          = 60*60
    cdef double dp          = 1.0/dt
    cdef int    tile_number = len(GRID['Xn'][0])
    cdef int    N           = len(STATVAR['T'])

    cdef cnp.ndarray[DTYPE_t, ndim=1] bp_lat     = np.zeros(tile_number)
    cdef cnp.ndarray[DTYPE_t, ndim=2] T_out      = np.zeros((N,tile_number))
    cdef cnp.ndarray[DTYPE_t, ndim=2] Water_out  = np.zeros((N,tile_number))
    cdef cnp.ndarray[DTYPE_t, ndim=2] Ice_out    = np.zeros((N,tile_number))
    cdef cnp.ndarray[DTYPE_t, ndim=1] Q_bot      = np.zeros(tile_number)
    cdef cnp.ndarray[DTYPE_t, ndim=1] TD         = np.zeros(tile_number)

    cdef double WaterDensity= 999.8 
    cdef double SnowDensity = 243
    cdef double Qgeo        = 0.05
    
    cdef cnp.ndarray[DTYPE_long, ndim=1] Xn = GRID['Xn'][0]
    cdef cnp.ndarray[DTYPE_t, ndim=1] dxp, dxn, dxs, Ao, Vp, cp, kp, Zp, Zs
    cdef cnp.ndarray[DTYPE_t, ndim=1] WaterIce, SolidMatrix, Water, Organic, temp_offset
    cdef cnp.ndarray[DTYPE_t, ndim=1] T, H, T_old, H_old, kp_rim, T_old_rim, Hinv_check, R, B
    cdef cnp.ndarray[DTYPE_t, ndim=1] kn, ks, anpn, anps, bp, ko, ap, lat_flux

    cdef double T0, temp_offset_max,  dxo, An, As,lic_lake, SnowDepthMax, SnowDepth
    cdef int sediment_idx, idx, ubc_idx, iter_count, n
    
    #-------------------------------------
    for i in reversed(range(tile_number)): #loop backward through tiles tile 1 is matrix    
        
        sediment_idx = int(max(0,WaterDepth[i]))
        n = N-sediment_idx
        if sediment_idx != 0:
            T0 = float(STATVAR['T'][sediment_idx-1,i]) 
        else:
            T0 = np.copy(T_air)
            
        if i == 0: #-- Tundra tile -> has snow
            SnowDepth = TEMP['SnowDepth'][0,0]
            SnowDepthMax= 0.5#PARA['SnowCoverMax'][0,0]
            Water, WaterIce, SnowDepth, sediment_idx \
                = SnowCover(STRAT['Water'][:,i],
                            STRAT['WaterIce'][:,i],
                            GRID['Zs'][:],
                            SnowDepth,SnowDepthMax, 
                            snow_fall,SnowDensity,WaterDensity,
                            current_date)
            n = N-sediment_idx
            dxp = GRID['dxp'][sediment_idx:]
            Zp  = GRID['Zp'][sediment_idx:]
            Zs  = GRID['Zs'][sediment_idx:]
            dxn = GRID['dxn'][sediment_idx:]
            dxs = GRID['dxs'][sediment_idx:]
            
            WaterIce    = WaterIce[sediment_idx:]
            Water       = Water[sediment_idx:]
            SolidMatrix = STRAT['SolidMatrix'][sediment_idx:,i]  
            Organic     = SolidMatrix*TotalC(Zp,location='tundra')
        else: #-- Pond tiles
            n = N-sediment_idx
            dxp = GRID['dxp'][sediment_idx:]
            Zp  = GRID['Zp'][sediment_idx:]
            Zs  = GRID['Zs'][sediment_idx:]
            dxn = GRID['dxn'][sediment_idx:]
            dxs = GRID['dxs'][sediment_idx:]
            
            WaterIce    = STRAT['WaterIce'][sediment_idx:,i]
            Water       = STRAT['Water'][sediment_idx:,i]
            SolidMatrix = STRAT['SolidMatrix'][sediment_idx:,i]  
            Organic     = SolidMatrix*TotalC(Zp,location='wb')
            
    
        dxo = GRID['dxo'][0,i] #[m]
        An  = GRID['An'][0,i]
        As  = GRID['As'][0,i]
        Ao  = GRID['Ao'][sediment_idx:,i]
        Vp  = GRID['Vp'][sediment_idx:,i]
        cp     = HeatCapacity(WaterIce, Water, SolidMatrix, Organic)
        kp     = ThermalConductivity(WaterIce, Water, SolidMatrix, Organic)
        kn, ks = Conductivities(dxp,dxn,dxs,kp)
        H,_    = Enthalpie(   STATVAR['T'][sediment_idx:,i], Water,    cp)
        T,fl   = EnthalpieInv(         H                   , WaterIce, cp)         
        H_old  = np.copy(H)
        T_old  = np.copy(T) 
        Water = fl*WaterIce
        
        # Add radiation to forcing temperature
        if i != 0:
            T0 += I_bot[i-1]/(cp[0]*dxn[0])
            
        if np.sum(SolidMatrix+WaterIce==0)>0:
            print('upper boundary zero in cells:')
            print(np.where(SolidMatrix+WaterIce==0))


        kp_rim    = TEMP['kp']  [sediment_idx:,0] #[W m-1 K-1]
        T_old_rim = STATVAR['T'][sediment_idx:,0]

        ubc_idx = 0
        
        temp_offset_max = 1
        iter_count = 1      
        while (temp_offset_max>=1e-4 or iter_count<8) and iter_count<=800:
            if iter_count>799:
                warnings.warn('did not reach convergence!', Warning)
                print(temp_offset_max)
    
            iter_count+=1
            cp = HeatCapacity(WaterIce, Water, SolidMatrix, Organic)
                            
            #pre-factores acording to grid cell sizes and thermal conductivities
            kn[ubc_idx] = kp[ubc_idx] # [W m-1 K-1] ensures full thermal conductivity at upper boundary
            anpn = An/Vp*kn/dxn
            anps = As/Vp*ks/dxs
            
            #Additional heat fluxes----------------------------------------
            bp = np.zeros(n)
            #flux from upper boundary
            bp[0] = (An/Vp[0]*kn[0]/dxn[0]) * T0 #[W/m³]

            #flux from lower boundary
            bp[n-1] = (An/Vp[n-1]) * Qgeo #[W/m³]

            if i!=0:
                ko = (kp_rim + kp)/2.0 #[W m-1 K-1] simple mean for testing lat flux 
                ap = anpn + anps + Ao/Vp*ko/dxo              
                ap[n-1] = anpn[n-1] + Ao[n-1]/Vp[n-1]*ko[n-1]/dxo
                anpn[0:ubc_idx+1] = 0.0
                
                #lateral heat flux (relative [W/m³])
                bp_lat = (Ao/Vp*ko/dxo) * T_old_rim 
                
                #update total lateral heat fluxes [W]
                lat_flux = (Ao*ko/dxo) * (T-T_old_rim) # [W]
               
            else:
                #for tile 1 the lateral flux is assumed to be a static
                #external flux reulting from the sum of the lateral heat
                #fluxes from the other tiles.
                ap = anpn + anps
                ap[n-1] = anpn[n-1]          
                anpn[0:ubc_idx+1] = 0.0 
                bp_lat = np.sum(TEMP['lat_flux'][sediment_idx:],axis=1)/Vp #[W/m³] 
            
            bp[:] += bp_lat       
            Hinv, _ = EnthalpieInv(H, WaterIce, cp) 
            _, dHdT = Enthalpie(Hinv, WaterIce, cp)
            
            Sp = -dp*dHdT
            Sc = dp*(H_old - H) - Sp*Hinv
    
            #TDMA solver --------------------------------------------------
            alpha = -anpn
            beta  = (ap - Sp)
            gamma = -anps
            delta = Sc + bp[:] # [W m-3]

            T = TDMAsolver(alpha,beta,gamma,delta) # [K, W m-3]
    
            #Nach unten gerichteter Wärmefluss; das Schmelzen von Eis benötigt Energie
            #(T-T_old)*rho*cp+ heat_of_fusion * (Ice_old-(WaterIce-Water))*rho_ice
            #--------------------------------------------------------------
            #update current state of H
            H = H + dHdT*(T - Hinv)            
            Hinv_check, fl = EnthalpieInv(H, WaterIce, cp)  
            Water = fl*WaterIce
            
            R = Hinv_check
            B = T

            temp_offset = np.abs(R-B)
            temp_offset_max = max(temp_offset[ubc_idx:-1])
            
        
        STRAT['WaterIce'][sediment_idx:,i] = WaterIce
        STRAT['SolidMatrix'][sediment_idx:,i] = SolidMatrix
        STRAT['Water'][sediment_idx:,i] = Water
        
        STATVAR['T'][sediment_idx:,i] = T.copy()
        STATVAR['H'][sediment_idx:,i] = H.copy()

        T_out[sediment_idx:,i] = T.copy()
        Water_out[sediment_idx:,i] = Water.copy()
        Ice_out[sediment_idx:,i] = WaterIce-Water
        
        TEMP['cp'][sediment_idx:,i] = cp
        TEMP['kp'][sediment_idx:,i] = kp #[W m-1 K-1]
        if i == 0:
            TEMP['SnowDepth'][0,0]= SnowDepth
            STATVAR['T'][:sediment_idx,0] = T_air
            
        # Compute Thaw Depth
        try:
            thaw_idx = int(np.argwhere(Water != 0)[-1])
            thaw_frac = Water[thaw_idx]/WaterIce[thaw_idx]
            TD[i] = max(0,thaw_frac*dxp[thaw_idx] + np.sum(dxp[:thaw_idx]))# - dxp[0])
        except:
            TD[i] = 0
            
        if i > 0:
            # Compute lateral heat flux in water
            
            # Bottom heat flux wie in FLake
            kappa_w     = 5.46E-01    # Molecular heat conductivity of water [J m^{-1} s^{-1} K^{-1}]
            Phi_B1      = 2           # B1 shape-function derivative 
            H_B1_min    = 1.0e-3      # Minimum thickness of the upper layer of bottom sediments [m]
            
            i_min = T.argmin()
            i_max = T.argmax()

            i_B1 = max([i_min,i_max])
            
            H_B1  = float(GRID['Zp'][i_B1])
            T_B1  = float(T[i_B1])

            Q_bot[i] =  - kappa_w*(T_B1-T0)/np.max([H_B1,H_B1_min])*Phi_B1
            
            kp_rim = ThermalConductivity(STRAT['WaterIce'][:sediment_idx,0],\
                                       STRAT['Water'][:sediment_idx,0],\
                                       STRAT['SolidMatrix'][:sediment_idx,0],
                                       STRAT['SolidMatrix'][:sediment_idx,0]*TotalC(GRID['Zp'][:sediment_idx],location='tundra'))  
            
            kp = ThermalConductivity(STRAT['WaterIce'][:sediment_idx,i],\
                                     STRAT['Water'][:sediment_idx,i],\
                                     STRAT['SolidMatrix'][:sediment_idx,i],
                                     STRAT['SolidMatrix'][:sediment_idx,i]*TotalC(GRID['Zp'][:sediment_idx],location='wb'))  
        
            ko = (kp_rim + kp)/2.0
            
            latflux_w = (GRID['Ao'][:sediment_idx,i]*ko/dxo)\
                        * (STATVAR['T'][:sediment_idx,i] -STATVAR['T'][:sediment_idx,0]) #[W]
            
            
            TEMP['lat_flux'][sediment_idx:,i] = lat_flux
            TEMP['lat_flux'][:sediment_idx,i] = latflux_w
            

    return T_out[:,:]+T_zero, Water_out[:,:], Ice_out, Q_bot ,TD,STRAT,STATVAR,TEMP, SnowDepth