#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed May  8 14:30:46 2019

@author: zoerehder

Initialize SoilHeat specifically for Samoylov Island
"""

import numpy as np
import xarray as xr 
from Cryo_forFlake_commonsurface_cy import HeatCapacity, Enthalpie, MakeGrid, ThermalConductivity, TotalC

def Porosity(depth, location, offset = 0):
    """
    Parameters
    ----------
    depth : [m] soil depth 
    location : 'tundra' or 'wb'

    Returns
    -------
    porosity : [frac.] Porosity approximation based on Zubrzycki et al. (2013) 
    """
    depth -= offset
    if location == 'tundra': # Polygonal Rims
        return 0.36*np.exp(-6.99 *depth) + 0.6
    elif location == 'wb':  #Polygonal centers
        x = 0.2
        return x*np.exp(-0.1 *depth) + 0.995 - x 
    elif location == 'iw': #ice-wedge
        return np.where(depth<1, 0.25*np.exp(-6.99 *depth) + 0.74,0.9) 
    else:
        raise ValueError('Not a valid location')
        


#%%
def initialize_soilheat(Z, waterbody , timesteps, 
                        depth_w  = None, 
                        A_wb     = None, 
                        pondtype = None):
    """
    Parameters
    ----------
    Z : [m] soil depth 
    waterbody : 'tundra' or 'wb'
    A_wb      : optional
    pondtype  : optional, either 'iw', 'pc' or 'mc', defaults to 'pc'
    
    Returns
    -------
    Initialized arrays for soil heat output
    GRID, PARA, STRAT, STATVAR, TEMP, cr_output
    """
#%%
    if np.all( pondtype == None):
        pondtype = len(waterbody)*['pc']
        
    if not np.sum(depth_w):
        depth_w = np.where(np.asarray(pondtype)=='pc', 0.6, pondtype)
        depth_w = np.where(np.asarray(depth_w) =='iw', 0.8, depth_w)
        depth_w = np.where(np.asarray(depth_w) =='mp', 1.2, depth_w).astype(float)  
        
    # --------- A - GRID ---------
    N = len(Z[:])-1 #number of grid cells
    n_waterbody = len(waterbody)+1 # add 1 tundra tile
    N_time = len(timesteps)
    
    kp = np.ones(N)*2.0 
    
    GRID = dict()
    GRID['Xn']  = np.asarray([range(n_waterbody)]) # number of tiles
    GRID['Xn'][0,0] = -1
    GRID['Xn'][0,1:] = waterbody
    GRID['Zp']  =Z[:-1]+np.diff(Z[:])/2 # grid cell midpoints
    GRID['dxp'] = np.diff(Z[:])         # grid cell sizes
    GRID['Zn']  = [0.0]*N 
    GRID['Zs']  = [0.0]*N
    GRID['dxn'] = [0.0]*N
    GRID['dxs'] = [0.0]*N

    GRID['Zn'], GRID['Zs'], GRID['dxn'], GRID['dxs'], kn, ks = MakeGrid(GRID['Zp'],GRID['dxp'],kp)
    
    
    
    if np.sum(A_wb != None):
        GRID['An']  = np.asarray([np.insert(A_wb, 0,4*np.sum(A_wb))])           # area of landscape unit [m²]
        GRID['dxo'] = np.asarray([np.insert(np.sqrt(A_wb)/np.pi,0, np.nan)])    # lateral distance between landscape units [m] 
        GRID['Ao']  = np.multiply(np.transpose([GRID['dxp']]),(2*GRID['An']/np.pi)**0.5) # lateral interface area of each layer [m²]

    else:
        # Total area of landscape unit [m2]
        An = np.where(np.asarray(pondtype)=='pc', 136677.36, pondtype)
        An = np.where(np.asarray(An)      =='iw', 41172.06,  An)
        An = np.where(np.asarray(An)      =='mp', 165818.70, An).astype(float)   
        GRID['An'] = np.asarray([np.insert(An, 0, 4*np.sum(An))])  
        
        # Mean distance to all other ponds [m]
        dxo = np.where(np.asarray(pondtype)=='pc', 784,  pondtype) 
        dxo = np.where(np.asarray(dxo)     =='iw', 754, dxo)
        dxo = np.where(np.asarray(dxo)     =='mp', 782, dxo).astype(float)   
        GRID['dxo'] = np.asarray([np.insert(dxo, 0, np.nan)])
        
        # Total boundary area
        Ao = np.where(np.asarray(pondtype)=='pc', 59074.85, pondtype) # total pond edge length [m]
        Ao = np.where(np.asarray(Ao)      =='iw', 14598.08,  Ao)
        Ao = np.where(np.asarray(Ao)      =='mp', 13353.14, Ao).astype(float)   
        GRID['Ao'] = np.asarray([np.insert(Ao, 0, np.nan)])      
        GRID['Ao']  = np.multiply(np.transpose([GRID['dxp']]),GRID['Ao']) # total edge area per layer [m2]
                                    
    GRID['As']  = GRID['An']
    GRID['Vp']  = np.multiply(np.transpose([GRID['dxp']]),GRID['An']) # volume of each layer [m³]
    
    # --------- B - PARAMETER ---------
    PARA= dict()
    PARA['Xn'] = GRID['Xn'] 
    
    PARA['WaterDepth']   = np.asarray([np.zeros((n_waterbody))]) #[m] water depth if there is a water body 
    PARA['WaterDensity'] = 1000             #[kg/m³]
    PARA['SnowDensity']  = 400              #[kg/m³]
    PARA['Qgeo']         = 0.05             #[W/m²] geothermal heat flux
    PARA['SnowCoverMax'] = np.asarray([0.5*np.ones((n_waterbody))]) # maximum snow cover thickness on landscape unit (if empty snow cover is limited by snowfall and Z domain only) 
    ##%%
    
    # --------- C - INITIALIZE VARIABLES ---------
    #Initialize soil stratigraphie, status variables, temporary variables   
    solidFraction = 0.4*np.ones((N,n_waterbody))
   
    STRAT = dict()#xr.Dataset()
    STRAT['Zp'] = GRID['Zp']
    STRAT['Xn'] = GRID['Xn'] 
    
    STATVAR = dict()#xr.Dataset()
    STATVAR['Zp'] = GRID['Zp']
    STATVAR['Xn'] = GRID['Xn'] 
    
    STRAT['WaterIce']    = np.zeros((N,n_waterbody)) # volume fraction Water+Ice
    STRAT['Water']       = np.zeros((N,n_waterbody)) # volume fraction liquid Water 
    STRAT['SolidMatrix'] = solidFraction # volume fraction other solid soil components
    STATVAR['T']         = np.zeros((N,n_waterbody))# [°C] status variable soil temperature 
    STATVAR['H']         = np.zeros((N,n_waterbody)) # [J/m^3] status variable enthalphy
    
    TEMP = dict()#xr.Dataset()
    TEMP['Zp'] = GRID['Zp']
    TEMP['Xn'] = GRID['Xn'] 
    TEMP['cp'] = np.zeros((N,n_waterbody))
    TEMP['kp'] = np.zeros((N,n_waterbody)) # [W m-1 K-1]
    TEMP['kn']  = kn
    TEMP['ks']  = ks
    TEMP['lat_flux']  = np.zeros((N,n_waterbody))
    TEMP['SnowDepth'] =  np.asarray([np.zeros((n_waterbody))])
            
    # Generell settings
    STRAT['Water'][:,:]  = 0
    STATVAR['T'][:,:]    = -3
    
    #  Tundra-Tile
    STRAT['SolidMatrix'][:,0] = np.where(STRAT['Zp']>0,
                                         1 - Porosity(STRAT['Zp'], 'tundra'),0)
    STRAT['WaterIce'][:,0]    =  np.where(STRAT['Zp']>0,
                                          (0.9-STRAT['SolidMatrix'][:,0]*1.1)
                                          *np.where(GRID['Zp']<=0.2,GRID['Zp']/0.2, 1),0)  
    TEMP['kp'][:,0] = ThermalConductivity(STRAT['WaterIce'][:,0],
                                          STRAT['Water'][:,0], 
                                          STRAT['SolidMatrix'][:,0], 
                                          STRAT['SolidMatrix'][:,0]*TotalC(GRID['Zp'],location='tundra'))
    TEMP['cp'][:,0] = HeatCapacity(STRAT['WaterIce'][:,0], 
                                   STRAT['Water'][:,0], 
                                   STRAT['SolidMatrix'][:,0], 
                                   STRAT['SolidMatrix'][:,0]*TotalC(GRID['Zp'],location='tundra'))
    STRAT['WaterIce'][:,0] = np.where(-STRAT['Zp'] < PARA['WaterDepth'][:,0], 
                                      STRAT['WaterIce'][:,0],0)  
    STRAT['WaterIce'][:,0] = np.where(STRAT['Zp']>0,
                                      STRAT['WaterIce'][:,0],0)
    STATVAR['H'][:,0],_    = Enthalpie(STATVAR['T'][:,0], 
                                       STRAT['WaterIce'][:,0], TEMP['cp'][:,0])   
    for i, t in enumerate(pondtype):
        wb = i+1
       
        if t == 'iw': # ice-wedge pond
            STRAT['SolidMatrix'][:,wb] = np.where(Z[1:] <= depth_w[wb-1],0,0.6)
            try:
                idx_sediment = int(np.argwhere( STRAT['SolidMatrix'][:,wb] == 0)[-1] +1)
            except: # no pond on top
                idx_sediment = 0
            PARA['WaterDepth']  [:,wb]              = idx_sediment 
            STRAT['SolidMatrix'][idx_sediment:,wb]  = 1 - Porosity(STRAT['Zp'][idx_sediment:]-STRAT['Zp'][idx_sediment], 'iw')
            STRAT['WaterIce']   [:,wb]              = (np.minimum(1-STRAT['SolidMatrix'][:,wb],1))
            TEMP['kp'][:,wb] = ThermalConductivity(STRAT['WaterIce'][:,wb], 
                                                   STRAT['Water'][:,wb], 
                                                   STRAT['SolidMatrix'][:,wb], 
                                                   STRAT['SolidMatrix'][:,wb]*TotalC(GRID['Zp'],location='wb'))
            TEMP['cp'][:,wb] = HeatCapacity(STRAT['WaterIce'][:,wb], 
                                            STRAT['Water'][:,wb], 
                                            STRAT['SolidMatrix'][:,wb], 
                                            STRAT['SolidMatrix'][:,wb]*TotalC(GRID['Zp'],location='wb'))

        elif t == 'pc' or t == 'mp': # polygonal-center and merged-polygonal pond
            STRAT['SolidMatrix'][:,wb] = np.where(Z[1:] <= depth_w[wb-1],0,0.6) # Set solidMatrix in water tiles to zero
            try:
                idx_sediment = int(np.argwhere( STRAT['SolidMatrix'][:,wb] == 0)[-1] +1)
            except: #no pond on top
                idx_sediment = 0
            PARA['WaterDepth'] [:,wb]               = idx_sediment  
            STRAT['SolidMatrix'][idx_sediment:,wb]   = 1 - Porosity(STRAT['Zp'][idx_sediment:], 'wb')
            STRAT['WaterIce'][:,wb]    = (np.minimum(1-STRAT['SolidMatrix'][:,wb]*1,1))
            TEMP['kp'][:,wb] = ThermalConductivity(STRAT['WaterIce'][:,wb], 
                                                   STRAT['Water'][:,wb], 
                                                   STRAT['SolidMatrix'][:,wb], 
                                                   STRAT['SolidMatrix'][:,wb]*TotalC(GRID['Zp'],location='wb'))
            TEMP['cp'][:,wb] = HeatCapacity(STRAT['WaterIce'][:,wb], 
                                            STRAT['Water'][:,wb], 
                                            STRAT['SolidMatrix'][:,wb], 
                                            STRAT['SolidMatrix'][:,wb]*TotalC(GRID['Zp'],location='wb'))

        else:
             print('WARNING: Could not identify pond type')
             return
        
        STRAT['WaterIce'][:,wb] = np.where(-STRAT['Zp'] < PARA['WaterDepth'][:,wb], 
                                           STRAT['WaterIce'][:,wb],0)  
        STRAT['WaterIce'][:,wb] = np.where(STRAT['Zp']>0,STRAT['WaterIce'][:,wb],0)
        STATVAR['H'][:,wb],_    = Enthalpie(STATVAR['T'][:,wb], 
                                            STRAT['WaterIce'][:,wb], 
                                            TEMP['cp'][:,wb])
#%%    
    cr_output = xr.Dataset({\
                'T_soil'    : (['Zp','Xn','time'],np.zeros((N,n_waterbody,N_time)),{'longname':'Temperatureprofile of underlying ground','unit':'K'}),\
                'W_soil'    : (['Zp','Xn','time'],np.zeros((N,n_waterbody,N_time)),{'longname':'Fraction of water in underlying ground','unit':'-'}),\
                'I_soil'    : (['Zp','Xn','time'],np.zeros((N,n_waterbody,N_time)),{'longname':'Fraction of ice in underlying ground','unit':'-'}),\
                'TD'        : (     ['Xn','time'],np.zeros((  n_waterbody,N_time)),{'longname':'Thaw depth', 'unit':'m'}),\
                'Q_bot'     : (     ['Xn','time'],np.zeros((  n_waterbody,N_time)),{'longname':'Heatflux from sediment to water', 'unit':'W m-2'}),\
                'Snow_depth': (           'time', np.zeros(                N_time),{'longname':'Snow Depth on tundra tile', 'unit':'m'})},\
                coords = {'Zp':GRID['Zp'], 'Xn':GRID['Xn'][0,:], 'time':timesteps})
    
    cr_output['T_soil'][:,:,0] = STATVAR['T']+273.15
    cr_output['W_soil'][:,:,0] = STRAT['Water']
    cr_output['I_soil'][:,:,0] = STRAT['WaterIce'] - STRAT['Water']

    return GRID, PARA, STRAT, STATVAR, TEMP, cr_output
    
